@@ -25,6 +25,9 @@
 #include <sys/socket.h>
 #include <netinet/in.h> /* For ntohl */
 #include <arpa/inet.h>
+
+#include <sys/mman.h>
+
 #else
 #include <windows.h>
 #define snprintf _snprintf
@@ -63,7 +66,7 @@
 		} \
 }
 
-const char GeoIP_country_code[251][3] = { "--","AP","EU","AD","AE","AF","AG","AI","AL","AM","AN",
+const char GeoIP_country_code[253][3] = { "--","AP","EU","AD","AE","AF","AG","AI","AL","AM","AN",
 	"AO","AQ","AR","AS","AT","AU","AW","AZ","BA","BB",
 	"BD","BE","BF","BG","BH","BI","BJ","BM","BN","BO",
 	"BR","BS","BT","BV","BW","BY","BZ","CA","CC","CD",
@@ -87,9 +90,10 @@ const char GeoIP_country_code[251][3] = { "--","AP","EU","AD","AE","AF","AG","AI
 	"TJ","TK","TM","TN","TO","TL","TR","TT","TV","TW",
 	"TZ","UA","UG","UM","US","UY","UZ","VA","VC","VE",
 	"VG","VI","VN","VU","WF","WS","YE","YT","RS","ZA",
-	"ZM","ME","ZW","A1","A2","O1","AX","GG","IM","JE"};
+	"ZM","ME","ZW","A1","A2","O1","AX","GG","IM","JE",
+  "BL","MF"};
 
-const char GeoIP_country_code3[251][4] = { "--","AP","EU","AND","ARE","AFG","ATG","AIA","ALB","ARM","ANT",
+const char GeoIP_country_code3[253][4] = { "--","AP","EU","AND","ARE","AFG","ATG","AIA","ALB","ARM","ANT",
 	"AGO","AQ","ARG","ASM","AUT","AUS","ABW","AZE","BIH","BRB",
 	"BGD","BEL","BFA","BGR","BHR","BDI","BEN","BMU","BRN","BOL",
 	"BRA","BHS","BTN","BV","BWA","BLR","BLZ","CAN","CC","COD",
@@ -113,9 +117,10 @@ const char GeoIP_country_code3[251][4] = { "--","AP","EU","AND","ARE","AFG","ATG
 	"TJK","TKL","TKM","TUN","TON","TLS","TUR","TTO","TUV","TWN",
 	"TZA","UKR","UGA","UM","USA","URY","UZB","VAT","VCT","VEN",
 	"VGB","VIR","VNM","VUT","WLF","WSM","YEM","YT","SRB","ZAF",
-	"ZMB","MNE","ZWE","A1","A2","O1","ALA","GGY","IMN","JEY"};
+	"ZMB","MNE","ZWE","A1","A2","O1","ALA","GGY","IMN","JEY",
+  "BLM","MAF"};
 
-const char * GeoIP_country_name[251] = {"N/A","Asia/Pacific Region","Europe","Andorra","United Arab Emirates","Afghanistan","Antigua and Barbuda","Anguilla","Albania","Armenia","Netherlands Antilles",
+const char * GeoIP_country_name[253] = {"N/A","Asia/Pacific Region","Europe","Andorra","United Arab Emirates","Afghanistan","Antigua and Barbuda","Anguilla","Albania","Armenia","Netherlands Antilles",
 	"Angola","Antarctica","Argentina","American Samoa","Austria","Australia","Aruba","Azerbaijan","Bosnia and Herzegovina","Barbados",
 	"Bangladesh","Belgium","Burkina Faso","Bulgaria","Bahrain","Burundi","Benin","Bermuda","Brunei Darussalam","Bolivia",
 	"Brazil","Bahamas","Bhutan","Bouvet Island","Botswana","Belarus","Belize","Canada","Cocos (Keeling) Islands","Congo, The Democratic Republic of the",
@@ -139,12 +144,13 @@ const char * GeoIP_country_name[251] = {"N/A","Asia/Pacific Region","Europe","An
 	"Tajikistan","Tokelau","Turkmenistan","Tunisia","Tonga","Timor-Leste","Turkey","Trinidad and Tobago","Tuvalu","Taiwan",
 	"Tanzania, United Republic of","Ukraine","Uganda","United States Minor Outlying Islands","United States","Uruguay","Uzbekistan","Holy See (Vatican City State)","Saint Vincent and the Grenadines","Venezuela",
 	"Virgin Islands, British","Virgin Islands, U.S.","Vietnam","Vanuatu","Wallis and Futuna","Samoa","Yemen","Mayotte","Serbia","South Africa",
-	"Zambia","Montenegro","Zimbabwe","Anonymous Proxy","Satellite Provider","Other","Aland Islands","Guernsey","Isle of Man","Jersey"};
+	"Zambia","Montenegro","Zimbabwe","Anonymous Proxy","Satellite Provider","Other","Aland Islands","Guernsey","Isle of Man","Jersey",
+  "Saint Barthelemy","Saint Martin"};
 
 /* Possible continent codes are AF, AS, EU, NA, OC, SA for Africa, Asia, Europe, North America, Oceania
 and South America. */
 
-const char GeoIP_country_continent[251][3] = {"--","AS","EU","EU","AS","AS","SA","SA","EU","AS","SA",
+const char GeoIP_country_continent[253][3] = {"--","AS","EU","EU","AS","AS","SA","SA","EU","AS","SA",
 	"AF","AN","SA","OC","EU","OC","SA","AS","EU","SA",
 	"AS","EU","AF","EU","AS","AF","AF","SA","AS","SA",
 	"SA","SA","AS","AF","AF","EU","SA","NA","AS","AF",
@@ -168,7 +174,8 @@ const char GeoIP_country_continent[251][3] = {"--","AS","EU","EU","AS","AS","SA"
 	"AS","OC","AS","AF","OC","AS","AS","SA","OC","AS",
 	"AF","EU","AF","OC","NA","SA","AS","EU","SA","SA",
 	"SA","SA","AS","OC","OC","OC","AS","AF","EU","AF",
-	"AF","EU","AF","--","--","--","EU","EU","EU","EU"};
+	"AF","EU","AF","--","--","--","EU","EU","EU","EU",
+  "SA","SA"};
 
 const char * GeoIPDBDescription[NUM_DB_TYPES] = {NULL, "GeoIP Country Edition", "GeoIP City Edition, Rev 1", "GeoIP Region Edition, Rev 1", "GeoIP ISP Edition", "GeoIP Organization Edition", "GeoIP City Edition, Rev 0", "GeoIP Region Edition, Rev 0","GeoIP Proxy Edition","GeoIP ASNum Edition","GeoIP Netspeed Edition","GeoIP Domain Name Edition"};
 
@@ -318,12 +325,17 @@ int _check_mtime(GeoIP *gi) {
 		if (stat(gi->file_path, &buf) != -1) {
 			if (buf.st_mtime != gi->mtime) {
 				/* GeoIP Database file updated */
-				if (gi->flags & GEOIP_MEMORY_CACHE) {
+				if (gi->flags & (GEOIP_MEMORY_CACHE | GEOIP_MMAP_CACHE)) {
+				    if ( gi->flags & GEOIP_MMAP_CACHE) {
+					munmap(gi->cache, gi->size);
+					gi->cache = NULL;
+				    } else {
 					/* reload database into memory cache */
 					if ((gi->cache = (unsigned char*) realloc(gi->cache, buf.st_size)) == NULL) {
 						fprintf(stderr,"Out of memory when reloading %s\n",gi->file_path);
 						return -1;
 					}
+				    }
 				}
 				/* refresh filehandle */
 				fclose(gi->GeoIPDatabase);
@@ -333,10 +345,20 @@ int _check_mtime(GeoIP *gi) {
 					return -1;
 				}
 				gi->mtime = buf.st_mtime;
-				if (gi->flags & GEOIP_MEMORY_CACHE) {
-					if (fread(gi->cache, sizeof(unsigned char), buf.st_size, gi->GeoIPDatabase) != (size_t) buf.st_size) {
-						fprintf(stderr,"Error reading file %s when reloading\n",gi->file_path);
-						return -1;
+				gi->size = buf.st_size;
+
+				if ( gi->flags & GEOIP_MMAP_CACHE) {
+				    gi->cache = mmap(NULL, buf.st_size, PROT_READ, MAP_PRIVATE, fileno(gi->GeoIPDatabase), 0);
+				    if ( gi->cache == MAP_FAILED ) {
+
+					    fprintf(stderr,"Error remapping file %s when reloading\n",gi->file_path);
+					    gi->cache = 0;
+					    return -1;
+				    }
+				} else if ( gi->flags & GEOIP_MEMORY_CACHE ) {
+				    if (fread(gi->cache, sizeof(unsigned char), buf.st_size, gi->GeoIPDatabase) != (size_t) buf.st_size) {
+					    fprintf(stderr,"Error reading file %s when reloading\n",gi->file_path);
+					    return -1;
 					}
 				}
 				if (gi->databaseSegments != NULL) {
@@ -437,39 +459,37 @@ unsigned int _GeoIP_seek_record (GeoIP *gi, unsigned long ipnum) {
 	return 0;
 }
 
-unsigned long _GeoIP_addr_to_num (const char *addr) {
-	int i;
-	char tok[4];
-	int octet;
-	int j = 0, k = 0;
-	unsigned long ipnum = 0;
-	char c = 0;
+unsigned long
+_GeoIP_addr_to_num(const char *addr)
+{
+	unsigned int    c, octet, t;
+	unsigned long   ipnum;
+	int             i = 3;
 
-	for (i=0; i<4; i++) {
-		for (;;) {
-			c = addr[k++];
-			if (c == '.' || c == '\0') {
-				tok[j] = '\0';
-				octet = atoi(tok);
-				if (octet > 255)
-					return 0;
-				ipnum += (octet << ((3-i)*8));
-				j = 0;
-				break;
-			} else if (c >= '0' && c<= '9') {
-				if (j > 2) {
-					return 0;
-				}
-				tok[j++] = c;
-			} else {
+	octet = ipnum = 0;
+	while ((c = *addr++)) {
+		if (c == '.') {
+			if (octet > 255)
 				return 0;
-			}
-		}
-		if(c == '\0' && i<3) {
-			return 0;
+			ipnum <<= 8;
+			ipnum += octet;
+			i--;
+			octet = 0;
+		} else {
+			t = octet;
+			octet <<= 3;
+			octet += t;
+			octet += t;
+			c -= '0';
+			if (c > 9)
+				return 0;
+			octet += c;
 		}
 	}
-	return ipnum;
+	if ((octet > 255) || (i != 0))
+		return 0;
+	ipnum <<= 8;
+	return ipnum + octet;
 }
 
 GeoIP* GeoIP_open_type (int type, int flags) {
@@ -524,7 +544,7 @@ GeoIP* GeoIP_open (const char * filename, int flags) {
 		free(gi);
 		return NULL;
 	} else {
-		if (flags & GEOIP_MEMORY_CACHE) {
+		if (flags & (GEOIP_MEMORY_CACHE | GEOIP_MMAP_CACHE) ) {
 			if (fstat(fileno(gi->GeoIPDatabase), &buf) == -1) {
 				fprintf(stderr,"Error stating file %s\n",filename);
 				free(gi->file_path);
@@ -532,8 +552,20 @@ GeoIP* GeoIP_open (const char * filename, int flags) {
 				return NULL;
 			}
 			gi->mtime = buf.st_mtime;
-			gi->cache = (unsigned char *) malloc(sizeof(unsigned char) * buf.st_size);
-			if (gi->cache != NULL) {
+			gi->size = buf.st_size;
+			/* MMAP added my Peter Shipley */
+			if ( flags & GEOIP_MMAP_CACHE) {
+			    gi->cache = mmap(NULL, buf.st_size, PROT_READ, MAP_PRIVATE, fileno(gi->GeoIPDatabase), 0);
+			    if ( gi->cache == MAP_FAILED ) {
+				fprintf(stderr,"Error mmaping file %s\n",filename);
+				free(gi->file_path);
+				free(gi);
+				return NULL;
+			    }
+			} else {
+			    gi->cache = (unsigned char *) malloc(sizeof(unsigned char) * buf.st_size);
+
+			    if (gi->cache != NULL) {
 				if (fread(gi->cache, sizeof(unsigned char), buf.st_size, gi->GeoIPDatabase) != (size_t) buf.st_size) {
 					fprintf(stderr,"Error reading file %s\n",filename);
 					free(gi->cache);
@@ -541,6 +573,7 @@ GeoIP* GeoIP_open (const char * filename, int flags) {
 					free(gi);
 					return NULL;
 				}
+			    }
 			}
 		} else {
 			if (flags & GEOIP_CHECK_CACHE) {
@@ -582,8 +615,14 @@ void GeoIP_delete (GeoIP *gi) {
 		return;
 	if (gi->GeoIPDatabase != NULL)
 		fclose(gi->GeoIPDatabase);
-	if (gi->cache != NULL)
+	if (gi->cache != NULL) {
+	    if ( gi->flags & GEOIP_MMAP_CACHE) {
+		munmap(gi->cache, gi->size);
+	    } else {
 		free(gi->cache);
+	    }
+	    gi->cache = NULL;
+	}
 	if (gi->index_cache != NULL)
 		free(gi->index_cache);
 	if (gi->file_path != NULL)
