@@ -53,6 +53,8 @@ GeoIPRecord * _extract_record(GeoIP* gi, unsigned int seek_record, int *next_rec
 	record = malloc(sizeof(GeoIPRecord));
 	memset(record, 0, sizeof(GeoIPRecord));
 
+	record->charset = gi->charset;
+
 	record_pointer = seek_record + (2 * gi->record_length - 1) * gi->databaseSegments[0];
 
 	if (gi->cache == NULL) {
@@ -70,6 +72,7 @@ GeoIPRecord * _extract_record(GeoIP* gi, unsigned int seek_record, int *next_rec
 	}
 
 	/* get country */
+	record->continent_code = (char *) GeoIP_country_continent[record_buf[0]];
 	record->country_code	= (char *) GeoIP_country_code [record_buf[0]];
 	record->country_code3 = (char *) GeoIP_country_code3[record_buf[0]];
 	record->country_name	= (char *) GeoIP_country_name [record_buf[0]];
@@ -89,8 +92,12 @@ GeoIPRecord * _extract_record(GeoIP* gi, unsigned int seek_record, int *next_rec
 	while (record_buf[str_length] != '\0')
 		str_length++;
 	if (str_length > 0) {
-		record->city = malloc(str_length+1);
-		strncpy(record->city, record_buf, str_length+1);
+		if ( gi->charset == GEOIP_CHARSET_UTF8 ) {
+			record->city = _iso_8859_1__utf8( (const char * ) record_buf );
+		} else {
+			record->city = malloc(str_length+1);
+			strncpy(record->city, ( const char * ) record_buf, str_length+1);
+		}
 	}
 	record_buf += (str_length + 1);
 	str_length = 0;
@@ -105,7 +112,7 @@ GeoIPRecord * _extract_record(GeoIP* gi, unsigned int seek_record, int *next_rec
 	record_buf += (str_length + 1);
 
 	/* get latitude */
-	for (j = 0; j < 3; ++j)
+for (j = 0; j < 3; ++j)
 		latitude += (record_buf[j] << (j * 8));
 	record->latitude = latitude/10000 - 180;
 	record_buf += 3;
@@ -205,3 +212,33 @@ void GeoIPRecord_delete (GeoIPRecord *gir) {
 	free(gir->postal_code);
 	free(gir);
 }
+
+
+
+char * _iso_8859_1__utf8(const char * iso) {
+	char c, k;
+	char * p;
+	char * t = iso;
+	int len = 0;
+	while ( ( c = *t++) ){
+		if ( c < 0 )
+			len++; 
+	}
+	len += t - iso;
+	t = p = malloc( len );
+	
+	if ( p ){
+		while ( ( c = *iso++ ) ) {
+			if (c < 0 ) {
+				k = 0xc2;
+				if (c >= -64 )
+					k++;
+				*t++ = k;
+				c &= ~0x40;
+			}
+			*t++ = c;
+		}
+		*t++ = 0x00;
+	}
+	return p;
+}
